import logging

import numpy as np
import pyqtgraph as pg
from qtpy import QtCore
from qtpy.QtCore import *
from qtpy.QtWidgets import *

from CIDAN.GUI.Data_Interaction.ROIExtractionThread import ROIExtractionThread
from CIDAN.GUI.Inputs.IntInput import IntInput
from CIDAN.GUI.Inputs.OptionInput import OptionInput
from CIDAN.GUI.ListWidgets.ROIListModule import ROIListModule
from CIDAN.GUI.ListWidgets.TrialListWidget import TrialListWidget
from CIDAN.GUI.SettingWidget.SettingsModule import roi_extraction_settings
from CIDAN.GUI.Tabs.Tab import Tab
from CIDAN.LSSC.functions.roi_extraction import combine_rois

logger1 = logging.getLogger("CIDAN.ROIExtractionTab")


class ROIExtractionTab(Tab):
    """Class controlling the ROI Extraction tab, inherits from Tab


        Attributes
        ----------
        main_widget : MainWidget
            A reference to the main widget
        data_handler : DataHandler
            A reference to the main DataHandler of MainWidget
        click_event : bool
            A bool that keeps track of whether a click event is currently happening used
            by roi_click_event and select_roi
        time_plot : pg.PlotWidget
            the plot for the time traces
        roi_list_module : ROIListModule
            The module the controlls the list of ROIs
        thread : ROIExtractionThread
            The thread that runs the roi extraction process
        foreground_slider : QSlider
            slider that determines intensity of background image

        """

    def __init__(self, main_widget):

        self.main_widget = main_widget
        self.data_handler = main_widget.data_handler
        self.click_event = False
        self.add_image = False
        self.outlines = True

        # part about selecting pixels

        self.select_pixel_on = False  # whether you can currently select pixels in the image
        self.brush_size = 1
        self.current_selected_pixels_list = []  # list of currently selected pixels in their 1d number format
        self.current_selected_pixels_mask = np.zeros((self.data_handler.shape[0],
                                                      self.data_handler.shape[
                                                          1]),
                                                     dtype=bool)  # mask 1 when selected 0 when not
        self.previous_values = {}
        self.select_pixel_color = [0, 255, 0]
        self.select_mode = "add"  # possibilities add and subtract from current selection
        # overload the image click and drag properites on the image item
        self.image_item = self.main_widget.roi_image_view.image_view.getImageItem()
        self.image_item.mouseClickEvent = lambda x: self.roi_view_click(x)
        self.image_item.mouseDragEvent = lambda x: self.roi_view_drag(x)

        # This part creates the top left settings/roi list view in two tabs
        self.tab_selector_roi = QTabWidget()
        self.tab_selector_roi.setStyleSheet("QTabWidget {font-size: 20px;}")

        # ROI modification Tab
        roi_modification_tab = QWidget()
        roi_modification_tab.setStyleSheet("margin:0px; padding: 0px;")

        roi_modification_tab_layout = QVBoxLayout()
        roi_modification_tab_layout.setContentsMargins(2, 2, 2, 2)
        roi_modification_tab.setLayout(roi_modification_tab_layout)
        self.roi_list_module = ROIListModule(main_widget.data_handler, self)
        roi_modification_tab_layout.addWidget(self.roi_list_module)
        roi_modification_button_top_layout = QHBoxLayout()
        roi_modification_tab_layout.addLayout(roi_modification_button_top_layout)

        add_new_roi = QPushButton(text="New ROI from Selection")
        add_new_roi.clicked.connect(lambda x: self.add_new_roi())
        add_to_roi = QPushButton(text="Add to ROI")
        add_to_roi.clicked.connect(
            lambda x: self.modify_roi(self.roi_list_module.current_selected_roi, "add"))

        sub_to_roi = QPushButton(text="Subtract from ROI")
        sub_to_roi.clicked.connect(
            lambda x: self.modify_roi(self.roi_list_module.current_selected_roi,
                                      "subtract"))
        delete_roi = QPushButton(text="Delete ROI")
        delete_roi.clicked.connect(
            lambda x: self.delete_roi(self.roi_list_module.current_selected_roi))

        # roi_modification_button_top_layout.addWidget(clear_from_selection)
        roi_modification_button_top_layout.addWidget(add_to_roi)
        roi_modification_button_top_layout.addWidget(sub_to_roi)
        roi_modification_button_top_layout.addWidget(add_new_roi)
        roi_modification_button_top_layout.addWidget(delete_roi)

        # roi_modification_button_top_layout.addWidget(subtract_from_roi)
        # painter_options = OptionInput("Pixel Selector Brush:", "",
        #                               lambda x,y: self.setSelectorBrushType(y),
        #                               default_index=0, tool_tip="",
        #                               val_list=["Off", "Add to Selection",
        #                                         "Subtract from Selection"])
        painter_button_group = QButtonGroup()
        off_button = QRadioButton(text="Off")
        off_button.setChecked(True)
        # off_button.connect(lambda x: self.setSelectorBrushType("off"))
        on_button = QRadioButton(text="Add to Selection")
        # on_button.connect(lambda x: self.setSelectorBrushType("Add to Selection"))
        sub_button = QRadioButton(text="Subtract from Selection")
        magic_wand = QRadioButton(text="Magic Wand")
        # sub_button.connect(lambda x: self.setSelectorBrushType("off"))
        painter_button_group.addButton(off_button)
        painter_button_group.addButton(on_button)
        painter_button_group.addButton(sub_button)
        painter_button_group.addButton(magic_wand)
        off_button.clicked.connect(
            lambda x: self.setSelectorBrushType("off"))
        on_button.clicked.connect(
            lambda x: self.setSelectorBrushType("add"))
        sub_button.clicked.connect(
            lambda x: self.setSelectorBrushType("subtract"))
        magic_wand.clicked.connect(
            lambda x: self.setSelectorBrushType("magic"))

        painter_layout = QHBoxLayout()
        painter_layout.addWidget(QLabel(text="Selector Brush: "))
        painter_layout.addWidget(off_button)
        painter_layout.addWidget(on_button)
        painter_layout.addWidget(sub_button)
        painter_layout.addWidget(magic_wand)
        roi_modification_tab_layout.addLayout(painter_layout)
        clear_from_selection = QPushButton(text="Clear Selection")
        clear_from_selection.clicked.connect(lambda x: self.clearPixelSelection())
        roi_modification_tab_layout.addWidget(clear_from_selection)
        self._brush_size_options = OptionInput("Brush Size:", "",
                                               lambda x, y: self.setBrushSize(y), 1,
                                         "Sets the brush size",
                                               ["1", "3", "5", "7", "9",
                                          "11", "15", "21", "27",
                                          "35"])

        roi_modification_tab_layout.addWidget(self._brush_size_options)

        # ROI Settings Tab
        process_button = QPushButton()
        process_button.setText("Apply Settings")
        self.thread = ROIExtractionThread(main_widget, process_button,
                                          self.roi_list_module, self)
        self.main_widget.thread_list.append(self.thread)
        process_button.clicked.connect(lambda: self.thread.runThread())
        self.roi_settings = QWidget()
        self.roi_settings_layout = QVBoxLayout()
        self.roi_settings_layout.setContentsMargins(2, 2, 2, 2)
        self.roi_settings.setLayout(self.roi_settings_layout)
        self.roi_settings_layout.addWidget(roi_extraction_settings(main_widget))
        self.roi_settings_layout.addWidget(process_button)

        # adding the tabs to the
        self.tab_selector_roi.addTab(self.roi_settings, "ROI Creation")
        self.tab_selector_roi.addTab(roi_modification_tab, "ROI Modification")
        if self.main_widget.dev:
            self.eigen_view = QWidget()
            self.eigen_view_layout = QVBoxLayout()
            self.eigen_view_box_input = IntInput("Box Number", "", None, 0, "", 0, 100,
                                                 1, False)
            self.eigen_view_number_input = IntInput("Vector Number", "", None, 0, "", 0,
                                                    100, 1, False)
            self.eigen_view_trial_input = IntInput("Trial Number", "", None, 0, "", 0,
                                                   100, 1, False)
            view_eigen_vector_button = QPushButton("View Eigen Vector")
            view_eigen_vector_button.clicked.connect(lambda x: self.view_eigen_vector())
            self.eigen_view_layout.addWidget(self.eigen_view_box_input)
            self.eigen_view_layout.addWidget(self.eigen_view_number_input)
            self.eigen_view_layout.addWidget(self.eigen_view_trial_input)
            self.eigen_view_layout.addWidget(view_eigen_vector_button)
            self.eigen_view.setLayout(self.eigen_view_layout)
            self.tab_selector_roi.addTab(self.eigen_view, "View Eigen Vectors")

        # Initialization of the background and rois
        self.current_foreground_intensity = 1
        self.set_background("", "Max Image", update_image=False)
        if self.main_widget.data_handler.rois_loaded:
            self.thread.endThread(True)

        # Image and time trace settings window

        display_settings_layout = QVBoxLayout()

        display_settings = QWidget()
        display_settings.setLayout(display_settings_layout)
        image_chooser = OptionInput("ROI Display type::", "",
                                    on_change_function=self.set_image,
                                    default_index=0,
                                    tool_tip="Choose background to display",
                                    val_list=["Outlines", "Blob"])

        display_settings_layout.addWidget(image_chooser)

        self.background_chooser = OptionInput("Background:", "",
                                              on_change_function=self.set_background,
                                              default_index=2,
                                              tool_tip="Choose background to display",
                                              val_list=["Blank Image", "Mean Image",
                                                        "Max Image",
                                                        # "Temporal Correlation Image",
                                                        "Eigen Norm Image"])

        display_settings_layout.addWidget(self.background_chooser)
        background_slider_layout = QHBoxLayout()
        background_slider_layout.addWidget(QLabel("0"))
        self.foreground_slider = QSlider(Qt.Horizontal)
        self.foreground_slider.setMinimum(0)

        self.foreground_slider.setMaximum(100)
        self.foreground_slider.setSingleStep(1)
        self.foreground_slider.valueChanged.connect(self.intensity_slider_changed)
        try:
            self.foreground_slider.setValue(80)
        except AttributeError:
            pass
        background_slider_layout.addWidget(self.foreground_slider)
        background_slider_layout.addWidget(QLabel("10"))

        display_settings_layout.addWidget(QLabel("Change foreground intensity:"))
        display_settings_layout.addLayout(background_slider_layout)

        # Initialize the tab with each column
        # TODO add way to select what time traces to display and calculate ones that aren't ready yet
        # this is to override how we do the column 2 to replace it with a split view
        tab_selector_time_trace = QTabWidget()
        tab_selector_time_trace.setStyleSheet("QTabWidget {font-size: 20px;}")
        tab_selector_time_trace.setMaximumHeight(220)
        self.time_plot = pg.PlotWidget()
        self.time_plot.showGrid(x=True, y=True, alpha=0.3)

        tab_selector_time_trace.addTab(self.time_plot, "Time Trace Plot")
        time_trace_settings = QWidget()
        time_trace_settings_layout = QVBoxLayout()
        time_trace_settings_layout.setContentsMargins(0, 0, 0, 0)

        # time_trace_settings.setStyleSheet("padding:0px; margin:2px;")
        time_trace_settings.setLayout(time_trace_settings_layout)
        self.time_trace_type = OptionInput("Time Trace Type", "",
                                           lambda x, y: x + y,
                                           default_index=0,
                                           tool_tip="Select way to calculate time trace",
                                           val_list=["Mean", "DeltaF/F"])
        time_trace_settings_layout.addWidget(self.time_trace_type,
                                             stretch=1)
        self._time_trace_trial_select_list = TrialListWidget()
        self._time_trace_trial_select_list.setMinimumHeight(115)
        self._time_trace_trial_select_list.set_items_from_list(
            self.data_handler.trials_all,
            self.data_handler.trials_loaded_time_trace_indices)
        time_trace_settings_layout.addWidget(self._time_trace_trial_select_list,
                                             stretch=5)
        time_trace_update_button = QPushButton("Update Time Traces")
        time_trace_settings_layout.addWidget(time_trace_update_button)
        time_trace_update_button.clicked.connect(
            lambda x: self.update_time_traces())
        tab_selector_time_trace.addTab(time_trace_settings, "Time Trace Settings")

        roi_view_tabs = QTabWidget()
        roi_view_tabs.setStyleSheet("QTabWidget {font-size: 20px;}")
        # roi_view_tabs.setStyleSheet(
        #     "QButton, QLabel, QSlider {padding: 5px; margin: 5px;}")
        # roi_view_tabs.setStyleSheet("QTabWidget {font-size: 20px;}")
        self.main_widget.roi_image_view.setStyleSheet(
            "margin:0px; border:0px  solid rgb(50, 65, "
            "75); padding: 0px;")
        roi_view_widget = QWidget()
        roi_view_widget_layout = QVBoxLayout()
        roi_view_widget_layout.setContentsMargins(0, 0, 0, 0)
        roi_view_widget_layout.addWidget(self.main_widget.roi_image_view)
        roi_view_widget.setLayout(roi_view_widget_layout)
        roi_view_tabs.addTab(roi_view_widget, "ROI Display")
        roi_view_tabs.addTab(display_settings, "Display Settings")
        self.column_2 = [roi_view_tabs, tab_selector_time_trace]
        super().__init__("ROI Extraction",
                         column_1=[self.tab_selector_roi],
                         column_2=self.column_2, column_2_display=True)
        # self.setStyleSheet("SettingsModule { border:1px solid rgb(50, 65, "
        #                    "75);} ")
        # column_2_split = QSplitter(Qt.Vertical)  # Layout for column 2 with split part
        # for module in self.column_2:
        #     column_2_split.addWidget(module)
        # column_2_split.setSizes([400, 100])
        #
        #
        # column_2_layout_box.addWidget(column_2_split)
        # # column_2_layout_box.addWidget(tab_selector_image)
        # self.layout.addLayout(column_2_layout_box)

    def setSelectorBrushType(self, type):

        if type == "off":
            self.select_pixel_on = False
            self.select_mode = type
        else:
            self.select_pixel_on = True
            self.select_mode = type

    def add_new_roi(self):
        """
        Adds a new roi using selection
        Returns
        -------

        """
        if len(self.current_selected_pixels_list) == 0:
            print("Please select some pixels")
            return
        shape = self.main_widget.data_handler.edge_roi_image_flat.shape
        self.data_handler.clusters.append(np.array(self.current_selected_pixels_list))
        self.data_handler.gen_roi_display_variables()
        self.data_handler.time_traces.append([])
        for _ in range(len(self.data_handler.trials_all)):
            self.data_handler.time_traces[-1].append(False)
        self.data_handler.calculate_time_trace(len(self.data_handler.clusters))
        self.update_roi()
        self.roi_list_module.set_list_items(self.data_handler.clusters)
        self.deselectRoiTime(2)

    def delete_roi(self, roi_num):
        """
        Deletes an roi
        Parameters
        ----------
        roi_num
            roi to delete starts at 1

        Returns
        -------

        """
        roi_num = roi_num - 1
        try:
            self.data_handler.clusters.pop(roi_num)
            self.data_handler.gen_roi_display_variables()
            self.data_handler.time_traces.pop(roi_num)
            self.update_roi()
            self.roi_list_module.set_list_items(self.data_handler.clusters)
            self.deselectRoiTime(2)
        except IndexError:
            print("Invalid ROI Selected")
    def modify_roi(self, roi_num, add_subtract="add"):
        """
        Add/subtracts the currently selected pixels from an ROI
        Parameters
        ----------
        roi_num roi to modify starting at 1
        add_subtract either add or subtract depending on operation wanted

        Returns
        -------
        Nothing
        """
        if roi_num is None:
            print("Please select an roi")
            return
        roi_num = roi_num - 1
        if len(self.current_selected_pixels_list) == 0:
            print("Please select some pixels")
            return
        if add_subtract == "add":
            print("Adding Selection to ROI #" + str(roi_num + 1))
            self.data_handler.clusters[roi_num] = combine_rois(
                self.data_handler.clusters[roi_num], self.current_selected_pixels_list)
            self.data_handler.gen_roi_display_variables()
            self.data_handler.calculate_time_trace(roi_num)
        if add_subtract == "subtract":
            print("Subtracting Selection from ROI #" + str(roi_num + 1))
            self.data_handler.clusters[roi_num] = [x for x in
                                                   self.data_handler.clusters[roi_num]
                                                   if
                                                   x not in self.current_selected_pixels_list]
            self.data_handler.gen_roi_display_variables()
            self.data_handler.calculate_time_trace(roi_num)
        self.update_roi()

    def update_roi(self):
        """Resets the roi image display"""
        shape = self.main_widget.data_handler.edge_roi_image_flat.shape
        self.data_handler.save_rois(self.data_handler.clusters)
        if self.outlines:
            self.roi_image_flat = np.hstack([self.data_handler.edge_roi_image_flat,
                                             np.zeros(shape),
                                             np.zeros(shape)])


        else:
            self.roi_image_flat = self.main_widget.data_handler.pixel_with_rois_color_flat

        self.select_image_flat = np.zeros([shape[0], 3])
        self.clearPixelSelection(update_display=False)
        self.updateImageDisplay()
    def draw(self, pos):
        pass

    def intensity_slider_changed(self):
        self.current_foreground_intensity = 10 - (
                float(self.foreground_slider.value()) / 10)
        self.updateImageDisplay()

    def set_background(self, name, func_name, update_image=True):
        # Background refers to the image behind the rois
        shape = self.main_widget.data_handler.shape
        if func_name == "Mean Image":
            self.current_background = self.main_widget.data_handler.mean_image.reshape(
                [-1, 1])
        if func_name == "Max Image":
            self.current_background = self.main_widget.data_handler.max_image.reshape(
                [-1, 1])
        if func_name == "Blank Image":
            self.current_background = np.zeros([shape[0] * shape[1], 1])
        # if func_name == "Temporal Correlation Image":
        #     self.current_background = self.data_handler.temporal_correlation_image.reshape(
        #         [-1, 1])
        if func_name == "Eigen Norm Image":
            self.current_background = self.data_handler.eigen_norm_image.reshape(
                [-1, 1])

        if update_image:
            self.updateImageDisplay()

    def set_image(self, name, func_name, update_image=True):
        # Background refers to the image behind the rois
        shape = self.main_widget.data_handler.edge_roi_image_flat.shape
        if func_name == "Outlines":
            self.outlines = True
            self.roi_image_flat = np.hstack([self.data_handler.edge_roi_image_flat,
                                             np.zeros(shape),
                                             np.zeros(shape)])
        if func_name == "Blob":
            self.outlines = False
            self.roi_image_flat = self.main_widget.data_handler.pixel_with_rois_color_flat

        if update_image:
            self.updateImageDisplay()

    def updateImageDisplay(self, new=False):
        try:
            # new is to determine whether the zoom should be saved
            # TODO add in update with image paint layer
            shape = self.main_widget.data_handler.shape
            if not hasattr(self, "select_image_flat"):
                self.select_image_flat = np.zeros([shape[0] * shape[1], 3])
            range_list = self.main_widget.roi_image_view.image_view.view.viewRange()
            background_max = self.current_background.max()
            background_image_scaled = (self.current_foreground_intensity * 255 / (
                background_max if background_max != 0 else 1)) * self.current_background
            background_image_scaled_3_channel = np.hstack(
                [background_image_scaled, background_image_scaled,
                 background_image_scaled])

            if new:
                # if self.add_image:
                combined = self.roi_image_flat + background_image_scaled_3_channel + self.select_image_flat

                # else:
                #     combined = background_image_scaled + self.select_image_flat
                #     mask = np.any(self.roi_image_flat != [0, 0, 0], axis=1)
                #     combined[mask] = self.roi_image_flat[mask]
                combined_reshaped = combined.reshape((shape[0], shape[1], 3))
                self.main_widget.roi_image_view.setImage(combined_reshaped)
                self.clearPixelSelection(update_display=False)
            else:
                self.image_item.image = background_image_scaled_3_channel.reshape(
                    (shape[0], shape[1], 3))
                self.image_item.updateImage(autoLevels=True)

                # if self.add_image:
                combined = (self.roi_image_flat + self.select_image_flat).reshape(
                    (shape[0], shape[1], 3))
                self.image_item.image += combined
                self.image_item.image[
                    self.current_selected_pixels_mask] += self.select_pixel_color

                # else:
                #     combined = self.select_image_flat+self.roi_image_flat
                #     combined_reshaped = combined.reshape((shape[1], shape[2], 3))
                #     mask = np.any(combined != [0, 0, 0], axis=1).reshape((shape[1], shape[2]))
                #
                #     self.image_item.image[mask] = combined_reshaped[mask]
                self.image_item.updateImage(autoLevels=False)

                # self.main_widget.roi_image_view.image_view.view.setRange(xRange=range_list[0],
                #                                                      yRange=range_list[1])
                # range_list = self.main_widget.roi_image_view.image_view.view.viewRange()
                # print(range_list)

            pass
        except AttributeError as e:
            logger1.error(e)

    def selectRoi(self, num):
        try:
            color_select = (245, 249, 22)
            color_roi = self.main_widget.data_handler.color_list[
                (num - 1) % len(self.main_widget.data_handler.color_list)]
            self.select_image_flat[
                self.main_widget.data_handler.clusters[num - 1]] = color_select
            self.updateImageDisplay()
        except AttributeError:
            pass

    def deselectRoi(self, num):

        color = self.main_widget.data_handler.color_list[
            (num - 1) % len(self.main_widget.data_handler.color_list)]
        shape_flat = self.data_handler.edge_roi_image_flat.shape
        self.select_image_flat[self.main_widget.data_handler.clusters[
            num - 1]] = color if not self.outlines \
            else np.hstack([self.data_handler.edge_roi_image_flat,
                            np.zeros(shape_flat),
                            np.zeros(shape_flat)])[
            self.main_widget.data_handler.clusters[num - 1]]
        self.updateImageDisplay()

    def selectRoiTime(self, num):
        try:
            color_roi = self.main_widget.data_handler.color_list[
                (num - 1) % len(self.main_widget.data_handler.color_list)]

            if (self.roi_list_module.roi_time_check_list[num - 1]):
                pen = pg.mkPen(color=color_roi, width=3)
                self.time_plot.plot(self.main_widget.data_handler.get_time_trace(num),
                                    pen=pen)
                self.time_plot.enableAutoRange(axis=0)
        except AttributeError:
            pass

    def deselectRoiTime(self, num):

        try:
            self.time_plot.clear()
            self.time_plot.enableAutoRange(axis=0)
            for num2, x in zip(range(1, len(self.roi_list_module.roi_time_check_list)),
                               self.roi_list_module.roi_time_check_list):
                if x:
                    color_roi = self.main_widget.data_handler.color_list[
                        (num2 - 1) % len(self.main_widget.data_handler.color_list)]

                    pen = pg.mkPen(color=color_roi, width=3)
                    self.time_plot.plot(
                        self.main_widget.data_handler.get_time_trace(num2), pen=pen)
        except AttributeError:
            print("No ROIs have been generated yet")

    def update_time_traces(self):
        if (self.data_handler.time_trace_params[
            "time_trace_type"] != self.time_trace_type.current_state()):
            self.data_handler.time_trace_params[
                "time_trace_type"] = self.time_trace_type.current_state()
            self.data_handler.calculate_time_traces()
        self.data_handler.update_selected_trials(
            self._time_trace_trial_select_list.selectedTrials())
        self.deselectRoiTime(0)
    def zoomRoi(self, num):
        """
        Zooms in to a certain roi
        Parameters
        ----------
        num : int
            roi num starts at 1

        Returns
        -------
        Nothing
        """
        num = num - 1

        max_cord = self.main_widget.data_handler.cluster_max_cord_list[num] + 15

        min_cord = self.main_widget.data_handler.cluster_min_cord_list[num] - 15

        self.main_widget.roi_image_view.image_view.getView().setXRange(min_cord[1],
                                                                       max_cord[1])
        self.main_widget.roi_image_view.image_view.getView().setYRange(min_cord[0],
                                                                       max_cord[0])

    def roi_view_click(self, event):
        if event.button() == QtCore.Qt.RightButton:
            if self.image_item.raiseContextMenu(event):
                event.accept()
        if hasattr(self.main_widget.data_handler, "pixel_with_rois_flat"):
            pos = event.pos()

            x = int(pos.x())
            y = int(pos.y())
            if self.select_mode == "magic":
                shape = self.data_handler.shape
                # self.clearPixelSelection(update_display=False)
                print("Generating ROI")
                new_roi = self.data_handler.genRoiFromPoint((x, y))
                if len(new_roi) == 0:
                    print(
                        "Please try again with a bigger growth factor or a different point, we couldn't find an roi where you last selected")
                    return

                for cord_1d in new_roi:
                    x_new, y_new = cord_1d // shape[1], cord_1d - (
                            cord_1d // shape[1]) * shape[1]
                    self.image_item.image[x_new, y_new] += [0, 255, 0]
                    self.current_selected_pixels_list.append(
                        shape[1] * x_new + y_new)
                    self.current_selected_pixels_mask[x_new, y_new] = True
                self.image_item.updateImage()
                print("Done generating ROI")
            elif self.select_pixel_on:
                event.accept()
                self.pixel_paint(x, y)

            else:
                self.click_event = True
                pixel_with_rois_flat = self.main_widget.data_handler.pixel_with_rois_flat
                shape = self.main_widget.data_handler.shape
                roi_num = int(pixel_with_rois_flat[shape[1] * x + y])
                # TODO change to int
                if roi_num != 0:
                    event.accept()
                    self.roi_list_module.set_current_select(roi_num)

    def roi_view_drag(self, event):
        # if event.button() == QtCore.Qt.RightButton:
        #     if self.image_item.raiseContextMenu(event):
        #         event.accept()
        if hasattr(self.main_widget.data_handler, "pixel_with_rois_flat"):
            pos = event.pos()

            x = int(pos.x())
            y = int(pos.y())
            if self.select_pixel_on and self.select_mode != "magic":
                event.accept()
                self.pixel_paint(x, y)

    def pixel_paint(self, x, y):
        try:
            shape = self.main_widget.data_handler.shape
            if self.select_mode == "add":


                for x_dif in range(self.brush_size * 2 + 1):
                    for y_dif in range(self.brush_size * 2 + 1):
                        x_new = x - self.brush_size - 1 + x_dif
                        y_new = y - self.brush_size - 1 + y_dif
                        if shape[1] * x_new + y_new \
                                not in self.current_selected_pixels_list:
                            self.image_item.image[x_new, y_new] += [0, 255, 0]
                            self.current_selected_pixels_list.append(
                                shape[1] * x_new + y_new)
                            self.current_selected_pixels_mask[x_new, y_new] = True

            if self.select_mode == "subtract":
                for x_dif in range(self.brush_size * 2 + 1):
                    for y_dif in range(self.brush_size * 2 + 1):
                        x_new = x - self.brush_size - 1 + x_dif
                        y_new = y - self.brush_size - 1 + y_dif
                        if shape[1] * x_new + y_new \
                                in self.current_selected_pixels_list:
                            self.image_item.image[x_new, y_new] -= [0, 255, 0]
                            self.current_selected_pixels_list.remove(
                                shape[1] * x_new + y_new)
                            self.current_selected_pixels_mask[x_new, y_new] = False

            self.image_item.updateImage()
        except IndexError:
            pass

        pass  # TODO use slicing to update pixel based on current thing

    def clearPixelSelection(self, update_display=True):
        shape = self.main_widget.data_handler.shape
        self.current_selected_pixels_mask = np.zeros([shape[0], shape[1]], dtype=bool)
        self.current_selected_pixels_list = []
        if update_display:
            self.updateImageDisplay()

    def check_pos_in_image(self, x, y):
        pass
        # TODO add in way to check if in image

    def setBrushSize(self, size):
        """
        Sets the brush size

        self.brush_size is the additional size on all dimensions in addition to middle
        point
        Parameters
        ----------
        size from option input

        Returns
        -------
        nothing
        """
        self.brush_size = int((int(size) - 1) / 2)

    def view_eigen_vector(self):
        vector_num = self.eigen_view_number_input.current_state()
        trial_num = self.eigen_view_trial_input.current_state()
        box_num = self.eigen_view_box_input.current_state()
        try:
            vector = self.data_handler.get_eigen_vector(box_num, trial_num=trial_num,
                                                        vector_num=vector_num)
            self.image_item.image = vector
            self.image_item.updateImage(autoLevels=True)
        except FileNotFoundError:
            print("Invalid location")



from functools import reduce
from typing import List, Tuple

import numpy as np
from dask import delayed
from scipy.ndimage.morphology import binary_fill_holes
from scipy.sparse import csr_matrix
from scipy.sparse.csgraph import connected_components as connected_components_graph
from skimage import measure

from CIDAN.LSSC.functions.embeddings import embedEigenSqrdNorm


@delayed
def roi_extract_image(*, e_vectors: np.ndarray,
                      original_shape: tuple, original_2d_vol: np.ndarray, merge: bool,
                      num_rois: int, refinement: bool, num_eigen_vector_select: int,
                      max_iter: int, roi_size_min: int, fill_holes: bool,
                      elbow_threshold_method: bool, elbow_threshold_value: float,
                      eigen_threshold_method: bool,
                      eigen_threshold_value: float, merge_temporal_coef: float,
                      roi_size_limit: int, box_num: int, initial_pixel=-1,
                      print_info=True) -> List[
    np.ndarray]:
    """
    Computes the Local Selective Spectral roi_extraction algorithm on an set of
    eigen vectors
    Parameters
    ----------
    e_vectors
        Eigen vector in 2D numpy array
    original_shape
        Original shape of image
    original_2d_vol
        A flattened 2d volume of the original image, used for mergestep
    num_rois
        Number of rois
    refinement
        If to do roi refinement step
    num_eigen_vector_select
        Number of eigen values to project into
    max_iter
        Max amount of pixels to try and roi around
    roi_size_min
        Min size for roi to be output if too big might limit number of rois
    roi_size_limit
        max size for rois
    fill_holes
        fills holes in rois
    elbow_threshold_method
        whether to use elbow thresholding of the rois in refinement step
    elbow_threshold_value
        The value to use for elbow threshold
    eigen_threshold_method
        whether to use thresholding method when selecting eigen values
    eigen_threshold_value
        value for eigen thresholding method
    box_num
        Box number, just used for print statements
    merge
        whether to merge different rois based spatial and temporal information
    merge_temporal_coef
        The coefficient limiting merging based of temporal information, 0 merge all
        1 merge none
    initial_pixel
        used if you want to generate an roi from a specific pixel
    Returns
    -------
    2D list of rois [[np.array of pixels roi 1], [
                          np.array  of pixels roi 2] ... ]
    It will have length num_rois unless max_iter amount is surpassed
    """
    if print_info:
        print("Spatial Box {}: Starting ROI selection process".format(box_num))
    pixel_length = e_vectors.shape[0]
    if len(original_shape) == 2:
        original_shape = (1, original_shape[0], original_shape[1])
    pixel_embedings = embedEigenSqrdNorm(
        e_vectors)  # embeds the pixels in the eigen space
    initial_pixel_list = np.flip(np.argsort(
        pixel_embedings))  # creates a list of pixels with the highest values
    # in the eigenspace this list is used to decide on the initial point
    # for the roi
    if initial_pixel != -1:
        initial_pixel_list = np.array([initial_pixel])
    roi_list = []  # output list of rois

    # iter_counter is used to limit the amount of pixels it tries
    # from initial_pixel_list
    iter_counter = 0
    total_counter = 0
    while len(roi_list) < num_rois and len(
            initial_pixel_list) > 0 and iter_counter < max_iter:
        iter_counter += 1
        total_counter += 1
        # print(iter_counter, len(roi_list),
        #       len(roi_list[-1]) if len(roi_list) > 0 else 0)
        initial_pixel = initial_pixel_list[0]  # Select initial point
        # select eigen vectors to project into
        small_eigen_vectors = select_eigen_vectors(e_vectors,
                                                   [initial_pixel],
                                                   num_eigen_vector_select,
                                                   threshold_method=
                                                   eigen_threshold_method,
                                                   threshold=eigen_threshold_value)
        # print("original",small_eigen_vectors.shape)
        # TODO Find way to auto determine threshold value automatically max values
        # project into new eigen space
        small_pixel_embeding_norm = embedEigenSqrdNorm(small_eigen_vectors)

        # calculate the distance between the initial point and each pixel
        # in the new eigen space
        small_pixel_distance = pixel_distance(small_eigen_vectors,
                                              initial_pixel)

        # selects pixels in roi
        pixels_in_roi = np.nonzero(
            small_pixel_distance <= small_pixel_embeding_norm)[0]

        # runs a connected component analysis around the initial point
        # in original image
        pixels_in_roi_comp = connected_component(pixel_length,
                                                 original_shape,
                                                 pixels_in_roi,
                                                 initial_pixel)

        pixels_in_roi_final = pixels_in_roi_comp

        # runs refinement step if enabled and if enough pixels in roi
        if refinement:  # TODO Look at this again
            # and len(pixels_in_roi_final) > \
            #  roi_size_threshold / 2
            # selects a new set of eigenvectors based on the pixels in roi
            rf_eigen_vectors = select_eigen_vectors(e_vectors,
                                                    pixels_in_roi_final,
                                                    num_eigen_vector_select,
                                                    threshold_method=eigen_threshold_method,
                                                    threshold=eigen_threshold_value)

            # print("rf",rf_eigen_vectors.shape)
            # embeds all pixels in this new eigen space
            rf_pixel_embedding_norm = embedEigenSqrdNorm(rf_eigen_vectors)

            # selects the initial point based on the pixel with max in
            # the new embedding space
            rf_initial_point = rf_select_initial_point(rf_pixel_embedding_norm,
                                                       pixels_in_roi_final)

            # calculate the distance between the initial point and each pixel
            # in the new eigen space
            rf_pixel_distance = pixel_distance(rf_eigen_vectors,
                                               rf_initial_point)

            # selects pixels in roi
            if elbow_threshold_method:
                threshold = elbow_threshold_value * elbow_threshold(rf_pixel_distance,
                                                                    np.argsort(
                                                                        rf_pixel_distance),
                                                                    half=True)
                rf_pixels_in_roi = np.nonzero(
                    rf_pixel_distance < threshold)[0]
            else:
                rf_pixels_in_roi = np.nonzero(
                    rf_pixel_distance <= rf_pixel_embedding_norm)[0]

            # runs a connected component analysis around the initial point
            # in original image
            rf_pixels_in_roi_comp = connected_component(pixel_length,
                                                        original_shape,
                                                        rf_pixels_in_roi,
                                                        rf_initial_point)
            rf_pixels_in_roi_filled = \
                fill_holes_func([rf_pixels_in_roi_comp], pixel_length, original_shape)[
                    0]
            pixels_in_roi_final = rf_pixels_in_roi_filled

        # checks if roi is big enough
        # print("roi size:", len(pixels_in_roi_final))
        # print( len(
        #         pixels_in_roi_final))
        if roi_size_min < len(
                pixels_in_roi_final) < roi_size_limit:
            roi_list.append(pixels_in_roi_final)
            iter_counter = 0
            # takes all pixels in current roi out of initial_pixel_list
            initial_pixel_list = np.extract(
                np.in1d(initial_pixel_list, pixels_in_roi_final,
                        assume_unique=True, invert=True),
                initial_pixel_list)
            if initial_pixel not in pixels_in_roi_final:
                initial_pixel_list = np.delete(initial_pixel_list, 0)

            # print(len(initial_pixel_list))
        else:
            # takes current initial point and moves it to end of
            # initial_pixel_list
            initial_pixel_list = np.delete(
                np.append(initial_pixel_list, initial_pixel_list[0]), 0)
    if fill_holes:
        # TODO combine into connected component function
        roi_list = fill_holes_func(roi_list, pixel_length, original_shape)
    # Merges rois
    if merge:
        roi_list = merge_rois(roi_list,
                              temporal_coefficient=merge_temporal_coef,
                              original_2d_vol=original_2d_vol)
        if fill_holes:
            roi_list = fill_holes_func(roi_list, pixel_length, original_shape)
    # print("Went through " + str(total_counter) + " iterations")
    return roi_list


def fill_holes_func(roi_list: List[np.ndarray], pixel_length: int,
                    original_shape: Tuple[int, int, int]) -> List[np.ndarray]:
    """
    Close holes in each roi
    Parameters
    ----------
    roi_list
        List of Rois in format: [[np.array of pixels roi 1],
        [np.array  of pixels roi 2] ... ]
    pixel_length
        Number of pixels in image
    original_shape
        Original shape of the image

    Returns
    -------
    roi list with holes filled in format: [[np.array of pixels roi 1],
        [np.array  of pixels roi 2] ... ]
    """
    for num, roi in enumerate(roi_list):
        original_zeros = np.zeros((pixel_length))
        original_zeros[roi] = 255
        image_filled = binary_fill_holes(np.reshape(original_zeros,
                                                    original_shape[1:]))
        image_filled_2d = np.reshape(image_filled, (-1))
        roi_list[num] = np.nonzero(image_filled_2d)[0]
    return roi_list


def pixel_distance(eigen_vectors: np.ndarray, pixel_num: int) -> np.ndarray:
    """
    Calculates squared distance between pixels in embedding space and initial_point
    Parameters
    ----------
    eigen_vectors
        The eigen vectors describing the vector space with
        dimensions number of pixels in image by number of eigen vectors
    pixel_num
        The number of the initial pixel in the eigen vectors

    Returns
    -------
    A np array with dim: number of pixels in image
    """
    return np.sum(np.power(eigen_vectors - eigen_vectors[pixel_num], 2),
                  axis=1)


def connected_component(pixel_length: int, original_shape: Tuple[int, int, int],
                        pixels_in_roi: np.ndarray,
                        initial_pixel_number: int) -> np.ndarray:
    """
    Runs a connected component analysis on a group of pixels in an image
    Parameters
    ----------
    pixel_length
        Number of pixels in image
    original_shape
        the original shape of image
    pixels_in_roi
        A list of pixels in the roi
    initial_pixel_number
        The number of the original pixel in the
        flattened image


    Returns
    -------
    An subset of the original pixels that are connected to initial pixel
    """
    # TODO add in im fill before connected component
    # first creates an image with pixel values of 1 if pixel in roi
    original_zeros = np.zeros(pixel_length)
    original_zeros[pixels_in_roi] = 1
    pixel_image = np.reshape(original_zeros, original_shape[1:])
    # runs connected component analysis on image
    blobs_labels = np.reshape(measure.label(pixel_image, background=0),
                              (-1))
    correct_label = blobs_labels[initial_pixel_number]

    # filters pixels to only ones with same label as initial pixel
    pixels_in_roi_new = np.nonzero(
        blobs_labels == correct_label)[0]
    return pixels_in_roi_new


def select_eigen_vectors(eigen_vectors: np.ndarray,
                         pixels_in_roi: np.ndarray,
                         num_eigen_vector_select: int,
                         threshold_method: bool = False,
                         threshold: float = .1) -> np.ndarray:
    """
    Selects eigen vectors that are most descriptive of a set a points
    Parameters
    ----------
    eigen_vectors
        The eigen vectors describing the vector space with
        dimensions number of pixels in image by number of eigen vectors
    pixels_in_roi
        Np array of indices of all pixels in roi
    num_eigen_vector_select
        Number of eigen vectors to select
    threshold_method
        this is a bool on whether to run the threshold method to select the eigen
        vectors threshold

    Returns
    -------
    the eigen vectors describing the new vector space with
        dimensions number of pixels in image by numb_eigen_vector_select

    """
    pixel_eigen_vec_values = np.abs(np.sum(eigen_vectors[pixels_in_roi], axis=0))
    pixel_eigen_vec_values_sort_indices = np.flip(
        np.argsort(pixel_eigen_vec_values))
    if threshold_method:
        threshold_filter = pixel_eigen_vec_values > threshold * \
                           pixel_eigen_vec_values[
                               pixel_eigen_vec_values_sort_indices[0]]
        small_eigen_vectors = eigen_vectors[:, np.nonzero(threshold_filter)[0]]

    if not threshold_method or small_eigen_vectors.shape[1] < num_eigen_vector_select:
        pixel_eigen_vec_values_sort_indices = np.flip(
            np.argsort(
                pixel_eigen_vec_values))
        small_eigen_vectors = eigen_vectors[:,
                              pixel_eigen_vec_values_sort_indices[
                              :num_eigen_vector_select]]
    return small_eigen_vectors


def rf_select_initial_point(pixel_embedings: np.ndarray,
                            pixels_in_roi: np.ndarray):
    """
    Selects an initial point for roi_extraction based on the pixels in current
    roi, this is part of the refinement step
    Parameters
    ----------
    pixel_embedings
        The embedings of each pixel in a vector space
    pixels_in_roi
        a list of the indices of the pixel in the roi

    Returns
    -------
    an indice for the initial point pixel
    """
    indice_in_roi = \
        np.flip(np.argsort(pixel_embedings[pixels_in_roi]))[0]
    return np.sort(pixels_in_roi)[indice_in_roi]


def elbow_threshold(pixel_vals: np.ndarray, pixel_val_sort_indices: np.ndarray,
                    half: bool = True) -> float:
    """
    Calculates the elbow threshold for the refinement step in roi_extraction algorithm,


    It determines the pixel that is farthest away from line drawn from the line from
    first to last or first to middle pixel. To find pixel it then projects each
    point(pixel # in sorted pixel_val, distance val) to the line then subtracts that
    from the points value to find distance from line

    Parameters
    ----------
    pixel_vals
        The distance values for each pixel
    pixel_val_sort_indices
        The array necessary to sort said pixels from lowest to highest
    half
        whether to run only with the closest half of the pixels, recommended

    Returns
    -------
    float, the optimal threshold based on elbow
    """
    n_points = len(pixel_vals) if not half else len(pixel_vals) // 2

    pixel_vals_sorted_zipped = np.array(list(
        zip(range(n_points), pixel_vals[pixel_val_sort_indices[:n_points]])))

    first_point = pixel_vals_sorted_zipped[0, :]
    last_point = pixel_vals_sorted_zipped[-1, :]
    line_vec = last_point - first_point

    line_vec_norm = line_vec / (np.sum(np.power(line_vec, 2)) ** .5)
    dist_from_first = pixel_vals_sorted_zipped - first_point
    proj_point_to_line = np.matmul(dist_from_first,
                                   line_vec_norm[:, None]) * line_vec_norm
    vec_to_line = dist_from_first - proj_point_to_line
    dist_to_line = np.power(np.sum(np.power(vec_to_line, 2), axis=1), .5)
    # fig, ax = plt.subplots(figsize=(4, 4))
    # ax.scatter(list(range(n_points)),
    #            dist_from_first[:n_points,1])
    # fig.savefig(
    #     "/Users/sschickler/Documents/LSSC-python/output_images/plots/dist_plot_{}2.png".format(num),
    #     aspect='auto')
    dist_max_indice = np.argmax(dist_to_line)
    threshold = pixel_vals_sorted_zipped[dist_max_indice][1]
    return threshold


def merge_rois(roi_list: List,
               temporal_coefficient: float, original_2d_vol: np.ndarray):
    # TODO is this the most efficient implementation I can do
    """
    Merges rois based on temporal and spacial overlap
    Parameters
    ----------
    roi_list
        List of Rois in format: [[np.array of pixels roi 1],
        [np.array  of pixels roi 2] ... ]
    temporal_coefficient
        The coefficient limiting merging based of temporal information, 0 merge all
        1 merge none
    original_2d_vol
        Volume of each pixel's time trace
    Returns
    -------
        List of new rois in format: [[np.array of pixels roi 1],
        [np.array  of pixels roi 2] ... ]
    """
    A = np.zeros([original_2d_vol.shape[0], len(roi_list)], dtype=bool)
    for num, cluster in enumerate(roi_list):
        A[cluster, num] = True
    A_graph = np.matmul(A.transpose(), A)
    A_csr = csr_matrix(A_graph)
    connected = connected_components_graph(A_csr, False, return_labels=True)
    roi_groups = [[] for _ in range(len(roi_list))]
    for num in range(len(roi_list)):
        roi_groups[connected[1][num]].append(roi_list[num])

    new_rois = []
    for group in roi_groups:
        group_zipped = list(enumerate(group))
        timetraces = [np.mean(original_2d_vol[roi], axis=0) for roi in group]
        while len(group_zipped) > 0:
            first_num, first_roi = group_zipped.pop(0)
            rois_to_merge = []
            for num, roi in enumerate(group_zipped):
                if compare_time_traces(timetraces[first_num],
                                       timetraces[roi[0]]) > temporal_coefficient:
                    rois_to_merge.append(num)
            first_roi = list(reduce(combine_rois,
                                    [first_roi] + [group_zipped[x][1] for x in
                                                   rois_to_merge]))
            for num in rois_to_merge[::-1]:
                group_zipped.pop(num)
            new_rois.append(np.array(first_roi))

    return new_rois


def compare_time_traces(trace_1: np.ndarray, trace_2: np.ndarray) -> float:
    """
    Compares two time traces based on pearson correlation

    Parameters
    ----------
    trace_1
        A 2d numpy list of values where 1 dimensions is each pixel in roi 1 and other is
        a time trace for each pixel
    trace_2
        A 2d numpy list of values where 1 dimensions is each pixel in roi 2 and other is
        a time trace for each pixel

    Returns
    -------
    the correlation as a float
    """
    trace_1_mean = np.mean(trace_1)
    trace_2_mean = np.mean(trace_2)
    trace_1_sub_mean = (trace_1 - trace_1_mean)
    trace_2_sub_mean = (trace_2 - trace_2_mean)
    top = np.dot(trace_1_sub_mean, trace_2_sub_mean)
    bottom = (np.dot(trace_1_sub_mean, trace_1_sub_mean) *
              np.dot(trace_2_sub_mean, trace_2_sub_mean)) ** .5
    return top / bottom


def combine_rois(roi1: List[int], roi2: List[int]) -> List[np.ndarray]:
    """
    Combines two lists of clusters into one
    Parameters
    ----------
    roi1
        One list of pixels in cluster each pixel # is based on 1d representation of
        image
    roi2
        List for other ROI

    Returns
    -------
    List of merged ROI
    """
    roi2_not_in_1 = list(filter(lambda x: x not in roi1, roi2))
    return np.array(list(roi1) + roi2_not_in_1)



from qtpy import QtCore
from qtpy.QtGui import *
from qtpy.QtWidgets import *

from CIDAN.GUI.ListWidgets.ROIItemModule import ROIItemModule
from CIDAN.GUI.ListWidgets.ROIItemWidget import ROIItemWidget


class ROIListModule(QFrame):
    def __init__(self, data_handler, roi_tab):
        super().__init__()
        self.current_selected_roi = 0

        self.roi_tab = roi_tab
        self.color_list = data_handler.color_list
        self.list = QListView()
        self.setStyleSheet("QListView::item { border-bottom: 1px solid rgb(50, 65, " +
                           "75); }")
        self.top_labels_layout = QHBoxLayout()
        label1 = QLabel(text="ROI Selected")
        label1.setMaximumWidth(100)
        self.top_labels_layout.addWidget(label1, alignment=QtCore.Qt.AlignRight)
        label2 = QLabel(text="ROI Num")
        label2.setMaximumWidth(100)
        self.top_labels_layout.addWidget(label2, alignment=QtCore.Qt.AlignLeft)
        self.top_labels_layout.addWidget(QLabel(), alignment=QtCore.Qt.AlignRight)
        self.top_labels_layout.addWidget(QLabel(), alignment=QtCore.Qt.AlignRight)
        self.top_labels_layout.addWidget(QLabel(), alignment=QtCore.Qt.AlignRight)
        self.top_labels_layout.addWidget(QLabel(), alignment=QtCore.Qt.AlignRight)
        label3 = QLabel(text="Time Trace On")
        label3.setMaximumWidth(100)
        self.top_labels_layout.addWidget(label3, alignment=QtCore.Qt.AlignRight)
        self.model = QStandardItemModel(self.list)
        self.list.setModel(self.model)
        self.layout = QVBoxLayout()
        self.layout.addLayout(self.top_labels_layout)
        self.layout.addWidget(self.list)
        self.roi_item_list = []
        self.setLayout(self.layout)

    def set_current_select(self, num):
        self.list.setCurrentIndex(self.model.index(int(num - 1), 0))
        self.roi_time_check_list[num - 1] = not self.roi_time_check_list[num - 1]
        self.roi_item_list[num - 1].select_check_box()
        self.roi_item_list[num - 1].select_time_check_box()

    def set_list_items(self, clusters):
        self.cluster_list = clusters
        for x in range(self.model.rowCount()):
            self.model.removeRow(0)
        self.roi_item_list = []
        for num in range(len(self.cluster_list)):
            item = ROIItemWidget(self.roi_tab,
                                 self.color_list[num % len(self.color_list)], self,
                                 num + 1)
            item1 = ROIItemModule(self.color_list[num % len(self.color_list)], num + 1,
                                  self.roi_tab)
            self.roi_item_list.append(item)
            self.model.appendRow(item1)
            self.list.setIndexWidget(item1.index(), item)
        self.roi_time_check_list = [False] * len(self.roi_item_list)
        self.roi_tab.tab_selector_roi.setCurrentIndex(1)
        # self.roi_item_list[0].select_check_box()
        # self.roi_item_list[0].select_time_check_box()
    # def change(self):
    #     # This is a way of running the select roi function when a checkbox is clicked there
    #     # needed to be a work around because can't just connect a signal to it
    #     for num, item, check_val in zip(range(1,len(self.roi_time_check_list)+1),self.roi_item_list,self.roi_time_check_list):
    #         if item.checkState() != check_val:
    #             self.roi_time_check_list[num-1] = item.checkState()
    #             if item.checkState():
    #                 self.roi_tab.selectRoi(num)
    #             else:
    #                 self.roi_tab.deselectRoi(num)



from qtpy.QtWidgets import *


class Input(QFrame):
    def __init__(self, display_name, program_name, on_change_function, default_val,
                 tool_tip, display_tool_tip=False, show_name=True):
        super().__init__()
        self.program_name = program_name
        self.input_box_1 = None
        self.tool_tip = tool_tip
        self.default_val = default_val
        self.display_name = display_name
        self.layout_main = QVBoxLayout()
        self.layout_main.setContentsMargins(2, 2, 2, 2)
        self.layout_h = QHBoxLayout()
        self.layout_h.setContentsMargins(0, 0, 0, 0)
        if show_name:
            temp_label = QLabel()
            temp_label.setText(display_name)
            temp_label.setToolTip(tool_tip)
            self.layout_h.addWidget(temp_label)
        self.on_change_function = on_change_function
        self.layout_main.addLayout(self.layout_h)
        if display_tool_tip:
            temp_lable_2 = QLabel()
            temp_lable_2.setText(tool_tip)
            self.layout_main.addWidget(temp_lable_2)

        self.setLayout(self.layout_main)

    def on_change(self, *args, **kwargs):
        try:
            if(not self.on_change_function==None):
                self.on_change_function(self.program_name, self.current_state())
        except AssertionError as e:
            print(e)
            self.set_default_val()

    def current_state(self):
        pass

    def set_default_val(self):
        pass

# TODO implement way to check spatial boxes square
